@@ -16,6 +16,8 @@
 
 #ifdef WITH_YAJL
 
+const char *base_offset=NULL;
+
 int json_add_argument(modsec_rec *msr, const char *value, unsigned length)
 {
     msc_arg *arg = (msc_arg *) NULL;
@@ -48,6 +50,8 @@ int json_add_argument(modsec_rec *msr, const char *value, unsigned length)
      */
     arg->value = apr_pstrmemdup(msr->mp, value, length);
     arg->value_len = length;
+    arg->value_origin_len = length;
+    arg->value_origin_offset = value-base_offset;
     arg->origin = "JSON";
 
     if (msr->txcfg->debuglog_level >= 9) {
@@ -146,6 +150,66 @@ static int yajl_number(void *ctx, const char *value, size_t length)
     return json_add_argument(msr, value, length);
 }
 
+static int yajl_start_array(void *ctx) {
+    modsec_rec *msr = (modsec_rec *) ctx;
+
+    if (!msr->json->current_key && !msr->json->prefix) {
+        msr->json->prefix = apr_pstrdup(msr->mp, "array");
+        msr->json->current_key = apr_pstrdup(msr->mp, "array");
+    }
+    else if (msr->json->prefix) {
+        msr->json->prefix = apr_psprintf(msr->mp, "%s.%s", msr->json->prefix,
+            msr->json->current_key);
+    }
+    else {
+        msr->json->prefix = apr_pstrdup(msr->mp, msr->json->current_key);
+    }
+    msr->json->current_depth++;
+    if (msr->json->current_depth > msr->txcfg->reqbody_json_depth_limit) {
+        msr->json->depth_limit_exceeded = 1;
+	return 0;
+    }
+
+    if (msr->txcfg->debuglog_level >= 9) {
+        msr_log(msr, 9, "New JSON hash context (prefix '%s')", msr->json->prefix);
+    }
+
+
+    return 1;
+}
+
+
+static int yajl_end_array(void *ctx) {
+    modsec_rec *msr = (modsec_rec *) ctx;
+    unsigned char *separator = (unsigned char *) NULL;
+
+    /**
+     * If we have no prefix, then this is the end of a top-level hash and
+     * we don't do anything
+     */
+    if (msr->json->prefix == NULL) return 1;
+
+    /**
+     * Current prefix might or not include a separator character; top-level
+     * hash keys do not have separators in the variable name
+     */
+    separator = strrchr(msr->json->prefix, '.');
+
+    if (separator) {
+        msr->json->prefix = apr_pstrmemdup(msr->mp, msr->json->prefix,
+            separator - msr->json->prefix);
+    }
+    else {
+        /**
+         * TODO: Check if it is safe to do this kind of pointer tricks
+         */
+        msr->json->prefix = (unsigned char *) NULL;
+    }
+    msr->json->current_depth--;
+
+    return 1;
+}
+
 /**
  * Callback for a new hash, which indicates a new subtree, labeled as the current
  * argument name, is being created
@@ -171,6 +235,11 @@ static int yajl_start_map(void *ctx)
     else {
         msr->json->prefix = apr_pstrdup(msr->mp, msr->json->current_key);
     }
+    msr->json->current_depth++;
+    if (msr->json->current_depth > msr->txcfg->reqbody_json_depth_limit) {
+        msr->json->depth_limit_exceeded = 1;
+	return 0;
+    }
 
     if (msr->txcfg->debuglog_level >= 9) {
         msr_log(msr, 9, "New JSON hash context (prefix '%s')", msr->json->prefix);
@@ -212,6 +281,7 @@ static int yajl_end_map(void *ctx)
         msr->json->current_key = msr->json->prefix;
         msr->json->prefix = (unsigned char *) NULL;
     }
+    msr->json->current_depth--;
 
     return 1;
 }
@@ -233,8 +303,8 @@ int json_init(modsec_rec *msr, char **error_msg) {
         yajl_start_map,
         yajl_map_key,
         yajl_end_map,
-        NULL /* yajl_start_array */,
-        NULL /* yajl_end_array  */
+        yajl_start_array,
+        yajl_end_array
     };
 
     if (error_msg == NULL) return -1;
@@ -250,6 +320,9 @@ int json_init(modsec_rec *msr, char **error_msg) {
     msr->json->prefix = (unsigned char *) NULL;
     msr->json->current_key = (unsigned char *) NULL;
 
+    msr->json->current_depth = 0;
+    msr->json->depth_limit_exceeded = 0;
+
     /**
      * yajl initialization
      *
@@ -273,12 +346,17 @@ int json_init(modsec_rec *msr, char **error_msg) {
 int json_process_chunk(modsec_rec *msr, const char *buf, unsigned int size, char **error_msg) {
     if (error_msg == NULL) return -1;
     *error_msg = NULL;
+    base_offset=buf;
 
     /* Feed our parser and catch any errors */
     msr->json->status = yajl_parse(msr->json->handle, buf, size);
     if (msr->json->status != yajl_status_ok) {
         /* We need to free the yajl error message later, how to do this? */
-        *error_msg = yajl_get_error(msr->json->handle, 0, buf, size);
+	if (msr->json->depth_limit_exceeded) {
+           *error_msg = "JSON depth limit exceeded";
+	} else {
+           *error_msg = yajl_get_error(msr->json->handle, 0, NULL, 0);
+	}
         return -1;
     }
 
@@ -298,7 +376,12 @@ int json_complete(modsec_rec *msr, char **error_msg) {
     msr->json->status = yajl_complete_parse(msr->json->handle);
     if (msr->json->status != yajl_status_ok) {
         /* We need to free the yajl error message later, how to do this? */
-        *error_msg = yajl_get_error(msr->json->handle, 0, NULL, 0);
+	if (msr->json->depth_limit_exceeded) {
+           *error_msg = "JSON depth limit exceeded";
+	} else {
+           *error_msg = yajl_get_error(msr->json->handle, 0, NULL, 0);
+	}
+
         return -1;
     }
 
