@@ -230,10 +230,20 @@ static char *construct_auditlog_filename(apr_pool_t *mp, const char *uniqueid) {
     char tstr[300];
     apr_size_t len;
 
+    /**
+     * This is required for mpm-itk & mod_ruid2, though should be harmless for other implementations 
+     * It also changes the return statement.
+     */
+    char *username;
+    apr_uid_t uid;
+    apr_gid_t gid;
+    apr_uid_current(&uid, &gid, mp);
+    apr_uid_name_get(&username, uid, mp);
+
     apr_time_exp_lt(&t, apr_time_now());
 
     apr_strftime(tstr, &len, 299, "/%Y%m%d/%Y%m%d-%H%M/%Y%m%d-%H%M%S", &t);
-    return apr_psprintf(mp, "%s-%s", tstr, uniqueid);
+    return apr_psprintf(mp, "/%s%s-%s", username, tstr, uniqueid);
 }
 
 /**
@@ -559,36 +569,9 @@ static void write_rule_json(modsec_rec *msr, const msre_rule *rule, yajl_gen g)
     if (rule->actionset->rev) {
         yajl_kv_string(g, "rev", log_escape(msr->mp, rule->actionset->rev));
     }
-    if (rule->actionset->msg) {
-        msc_string *var = (msc_string *)apr_palloc(msr->mp, sizeof(msc_string));
-        var->value = (char *)rule->actionset->msg;
-        var->value_len = strlen(rule->actionset->msg);
-        expand_macros(msr, var, NULL, msr->mp);
-
-        yajl_kv_string(g, "msg", log_escape_ex(msr->mp, var->value, var->value_len));
-    }
     if (rule->actionset->version) {
         yajl_kv_string(g, "version", log_escape(msr->mp, rule->actionset->version));
     }
-    if (rule->actionset->logdata) {
-        char *logdata = NULL;
-        msc_string *var = (msc_string *)apr_pcalloc(msr->mp, sizeof(msc_string));
-        var->value = (char *)rule->actionset->logdata;
-        var->value_len = strlen(rule->actionset->logdata);
-        expand_macros(msr, var, NULL, msr->mp);
-
-        logdata = apr_pstrdup(msr->mp, log_escape_hex(msr->mp, (unsigned char *)var->value, var->value_len));
-
-        // if it is > 512 bytes, then truncate at 512 with ellipsis.
-        if (strlen(logdata) > 515) {
-            logdata[512] = '.';
-            logdata[513] = '.';
-            logdata[514] = '.';
-            logdata[515] = '\0';
-        }
-
-        yajl_kv_string(g, "logdata", logdata);
-    }
     if (rule->actionset->severity != NOT_SET) {
         yajl_kv_int(g, "severity", rule->actionset->severity);
     }
@@ -905,7 +888,7 @@ void sec_audit_logger_json(modsec_rec *msr) {
                 for(i = 0; i < tarr->nelts; i++) {
                     msc_arg *arg = (msc_arg *)telts[i].val;
                     if (arg->origin != NULL &&
-                            strcmp(arg->origin, "BODY") != 0)
+                            ( strcmp(arg->origin, "BODY") != 0 && strcmp(arg->origin, "JSON") !=0) )
                         continue;
 
                     if (last_offset == 0) { /* The first time we're here. */
@@ -1009,6 +992,7 @@ void sec_audit_logger_json(modsec_rec *msr) {
 
                         /* Write the sanitized chunk to the log
                          * and advance to the next chunk. */
+                        chunk->data[chunk->length] = 0;
                         yajl_string(g, chunk->data);
                         chunk_offset += chunk->length;
                     }
@@ -1183,21 +1167,33 @@ void sec_audit_logger_json(modsec_rec *msr) {
         }
 
         /* Apache-Handler */
+#ifdef LOG_NO_HANDLER
+        if (msr->txcfg->debuglog_level >= 9)
+#endif
         if (msr->r->handler != NULL) {
             yajl_kv_string(g, "handler", msr->r->handler);
         }
 
 
         /* Stopwatch2 */
+#ifdef LOG_NO_STOPWATCH
+        if (msr->txcfg->debuglog_level >= 9)
+#endif
         format_performance_variables_json(msr, g);
 
         /* Our response body does not contain chunks */
         /* ENH Only write this when the output was chunked. */
         /* ENH Add info when request body was decompressed, dechunked too. */
+#ifdef LOG_NO_DECHUNK
+        if (msr->txcfg->debuglog_level >= 9)
+#endif
         if (wrote_response_body) {
             yajl_kv_bool(g, "response_body_dechunked", 1);
         }
 
+#ifdef LOG_NO_SERVER_CONTEXT
+        if (msr->txcfg->debuglog_level >= 9) {
+#endif
         sec_auditlog_write_producer_header_json(msr, g);
 
         /* Server */
@@ -1292,6 +1288,9 @@ void sec_audit_logger_json(modsec_rec *msr) {
         if (been_opened == 1) {
             yajl_gen_map_close(g); // sanitized args map is finished
         }
+#ifdef LOG_NO_SERVER_CONTEXT
+	}
+#endif
 
         /* Web application info. */
         if ( ((msr->txcfg->webappid != NULL)&&(strcmp(msr->txcfg->webappid, "default") != 0))
@@ -1364,10 +1363,12 @@ void sec_audit_logger_json(modsec_rec *msr) {
         for(cfiles = 0; cfiles < msr->mpd->parts->nelts; cfiles++) {
             if (parts[cfiles]->type == MULTIPART_FILE) {
                 if(parts[cfiles]->filename != NULL) {
+                    yajl_gen_map_open(g);
                     yajl_kv_int(g, "file_size", parts[cfiles]->tmp_file_size);
                     yajl_kv_string(g, "file_name", log_escape(msr->mp, parts[cfiles]->filename));
                     yajl_kv_string(g, "content_type", parts[cfiles]->content_type ? parts[cfiles]->content_type : "<Unknown Content-Type>");
                     total_size += parts[cfiles]->tmp_file_size;
+                    yajl_gen_map_close(g);
                 }
             }
         }
@@ -1448,11 +1449,12 @@ void sec_audit_logger_json(modsec_rec *msr) {
     yajl_gen_clear(g);
     yajl_gen_free(g);
 
+    sec_auditlog_write(msr, "\n", 1);
+
     /* Return here if we were writing to a serial log
      * as it does not need an index file.
      */
     if (msr->txcfg->auditlog_type != AUDITLOG_CONCURRENT) {
-        sec_auditlog_write(msr, "\n", 1);
 
         /* Unlock the mutex we used to serialise access to the audit log file. */
         rc = apr_global_mutex_unlock(msr->modsecurity->auditlog_lock);
@@ -1748,7 +1750,7 @@ void sec_audit_logger_native(modsec_rec *msr) {
                 for(i = 0; i < tarr->nelts; i++) {
                     msc_arg *arg = (msc_arg *)telts[i].val;
                     if (arg->origin != NULL &&
-                            strcmp(arg->origin, "BODY") != 0)
+                            ( strcmp(arg->origin, "BODY") != 0 && strcmp(arg->origin, "JSON") != 0) )
                         continue;
 
                     if (last_offset == 0) { /* The first time we're here. */
@@ -1895,7 +1897,7 @@ void sec_audit_logger_native(modsec_rec *msr) {
 
         /* There are no response headers (or the status line) in HTTP 0.9 */
         if (msr->response_headers_sent) {
-            if (msr->status_line != NULL) {
+            if (msr->status_line != NULL && msr->status_line[0] != '\0') {
                 text = apr_psprintf(msr->mp, "%s %s\n", msr->response_protocol,
                         msr->status_line);
             } else {
@@ -1998,12 +2000,18 @@ void sec_audit_logger_native(modsec_rec *msr) {
         }
 
         /* Apache-Handler */
-        if (msr->r->handler != NULL) {
+#ifdef LOG_NO_HANDLER
+	if (msr->txcfg->debuglog_level >= 9)
+#endif
+	if (msr->r->handler != NULL) {
             text = apr_psprintf(msr->mp, "Apache-Handler: %s\n", msr->r->handler);
             sec_auditlog_write(msr, text, strlen(text));
         }
 
         /* Stopwatch; left in for compatibility reasons */
+#ifdef LOG_NO_STOPWATCH
+	if (msr->txcfg->debuglog_level >= 9) {
+#endif
         text = apr_psprintf(msr->mp, "Stopwatch: %" APR_TIME_T_FMT " %" APR_TIME_T_FMT " (- - -)\n",
             msr->request_time, (now - msr->request_time));
         sec_auditlog_write(msr, text, strlen(text));
@@ -2016,15 +2024,24 @@ void sec_audit_logger_native(modsec_rec *msr) {
                 "; %s\n", msr->request_time, (now - msr->request_time), perf_all);
             sec_auditlog_write(msr, text, strlen(text));
         }
+#ifdef LOG_NO_STOPWATCH
+        }
+#endif
 
         /* Our response body does not contain chunks */
         /* ENH Only write this when the output was chunked. */
         /* ENH Add info when request body was decompressed, dechunked too. */
+#ifdef LOG_NO_DECHUNK
+	if (msr->txcfg->debuglog_level >= 9)
+#endif
         if (wrote_response_body) {
             text = apr_psprintf(msr->mp, "Response-Body-Transformed: Dechunked\n");
             sec_auditlog_write(msr, text, strlen(text));
         }
 
+#ifdef LOG_NO_SERVER_CONTEXT
+        if (msr->txcfg->debuglog_level >= 9) {
+#endif
         sec_auditlog_write_producer_header(msr);
 
         /* Server */
@@ -2093,8 +2110,11 @@ void sec_audit_logger_native(modsec_rec *msr) {
                 sec_auditlog_write(msr, text, strlen(text));
             }
         }
+#ifdef LOG_NO_SERVER_CONTEXT
+	}
+#endif
 
-        /* Web application info. */
+	/* Web application info. */
         if ( ((msr->txcfg->webappid != NULL)&&(strcmp(msr->txcfg->webappid, "default") != 0))
             || (msr->sessionid != NULL) || (msr->userid != NULL))
         {
